import paho.mqtt.client as mqtt
import json
from konker_client.client import Client


class Mqtt(Client):

    def __init__(self, user, password):
        self.client = mqtt.Client()
        self.client.on_connect = self.on_connect
        self.client.on_message = self.on_message
        self.client.on_disconnect = self.on_disconnect
        self.is_connected = False
        self.client.username_pw_set(user, password)
        self.client.connect('mqtt.demo.konkerlabs.net', 1883)
        self.client.loop_start()

    def send_message(self, channel: str, message: dict):
        message = json.dumps(message)
        self.client.publish(channel, message)

    def on_connect(self, client, userdata, flags, rc):
        self.is_connected = True
        if self.is_connected:
            print("Conectado com sucesso, aguardando mensagem.")
            self.client.subscribe("sub/426psued49pa/temp")

    def on_disconnect(self, client, userdata, rc):
        self.is_connected = False

    def on_message(self, client, data, msg):
        print(msg.topic + " " + str(msg.payload))

    def on_subscribe(self, client, userdata, mid, granted_qos):
        print(f'Subscribe: {userdata}')



from konker_client.client import Client
from timeit import default_timer as timer
import requests
import logging
import json
import asyncio
from aiohttp import ClientSession


class Request(Client):

    def __init__(self, user: str, password: str):
        """Constructor Method for the Request Class

        :param user: The devices username
        :type user: str
        :param password: The devices password
        :type password: str
        """
        self._logger = logging.getLogger(__name__)
        self.url_konker_post = 'http://data.demo.konkerlabs.net:80/pub/'
        self.user = user
        self.password = password

    def send_message(self, channel: str, message: dict) -> bool:
        """Method for posting messages to the Konker Plataform

        :param user: The devices username
        :type user: str
        :param password: The devices password
        :type password: str
        """
        method_start_time = timer()
        self._logger.info("Method post_konker started")

        url = f'{self.url_konker_post}{self.user}/{channel}'
        headers = {'content-type': 'application/json', 'Accept': 'application/json'}
        auth = (self.user, self.password)

        try:
            response = requests.post(url, headers=headers, auth=auth, data=json.dumps(message))
        except Exception:
            method_end_time = timer()
            total_seconds = method_end_time - method_start_time
            self._logger.error(f"Method post_konker finished in {total_seconds:.2} seconds with error", exc_info=1)
            return False

        method_end_time = timer()
        total_seconds = method_end_time - method_start_time
        if response.status_code != 200:
            self._logger.error(f"Method post_konker finished in {total_seconds:.2} seconds with error status != 200")
            return False
        else:
            self._logger.info(f"Method post_konker finished in {total_seconds:.2} seconds")
            return True

    def send_messages_assync(self, channel: str, message: dict):
        # TODO
        pass


import datetime
import time
import platform
import psutil
import netifaces
import os


class System:
    """
    Classe que define o System
    """
    def __init__(self):
        self.hostname = None
        self.ip = None
        self.host_memory_perc = None
        self.host_memory_used = None
        self.host_swap_perc = None
        self.host_swap_used = None
        self.host_cpu = None
        self.host_disks = None
        self.process_memory_perc = None
        self.process_memory_used = None
        self.process_swap_perc = None
        self.process_swap_used = None
        self.process_cpu = None

        self.pid = os.getpid()

    def get_health(self):
        self.hostname = platform.uname()[1]
        self.ip = self.get_ip()
        self.host_memory_perc, self.host_memory_used = self.get_host_memory()
        self.host_swap_perc, self.host_swap_used = self.get_host_swap()
        self.host_cpu = self.get_host_cpu()
        self.host_disks = self.get_host_disks()
        self.process_memory_perc, self.process_memory_used = self.get_process_memory()
        self.process_swap_perc, self.process_swap_used = self.get_process_swap()
        self.process_cpu = self.get_process_cpu()

    def get_process(self) -> psutil.Process:
        """
        Function that returns the Process
        """
        return psutil.Process(self.pid)

    def get_process_swap(self) -> tuple:
        """
        Function that collects and returns the percentage of process and use of the Memory Swap
        """
        process = self.get_process()
        perc = 0
        usage = 0
        if psutil.LINUX and psutil.swap_memory().total > 0:
            perc = (process.memory_full_info().swap /
                    psutil.swap_memory().total) * 100
            usage = process.memory_full_info().swap
        return perc, usage

    def get_process_cpu(self):
        """
        Função que coleta e retorna o percentual de processo de CPU
        """
        pid = os.getpid()
        processo = psutil.Process(pid)
        perc = round(processo.cpu_percent(), 2)
        return perc

    def get_process_memory(self):
        """
        Função que coleta e retorna o percentual de processo e de uso da Memoria
        """
        pid = os.getpid()
        processo = psutil.Process(pid)
        perc = round(processo.memory_percent() / (10**6), 2)
        usage = round(((psutil.virtual_memory().total / (10**6)) * processo.memory_percent()) / 100, 2)
        return perc, usage

    def get_host_cpu(self):
        """
        Função que retorna o percentual de utilização do CPU
        """
        perc = round(psutil.cpu_percent(), 2)
        return perc

    def get_host_memory(self):
        """
        Função que retorna o percentual e uso da memoria
        """
        perc = round(psutil.virtual_memory().percent, 2)
        usage = round(((psutil.virtual_memory().total / (10**6)) * psutil.virtual_memory().percent) / 100, 2)
        return perc, usage

    def get_host_swap(self):
        """
        Função que retorna o percentual e uso da memoria swap
        """
        perc = round(psutil.swap_memory().percent, 2)
        usage = round(psutil.swap_memory().used / (10**6), 2)
        return perc, usage

    def get_ip(self):
        """
        Função que coleta e retorna os IPs
        """
        interfaces = netifaces.interfaces()
        for i in interfaces:
            if i == 'lo':
                continue
            iface = netifaces.ifaddresses(i).get(netifaces.AF_INET)
            if iface is not None:
                return iface[0]['addr']

    def get_host_disks(self):
        """
        Função que coleta e retorna as informações de disco
        """
        resposta = []
        # disk.device para pegar o nome mesmo /dev/sdaX
        for disk in psutil.disk_partitions(all=False):
            if disk.opts == 'cdrom':
                continue
            # print("Mountpoint: ", disk.mountpoint)
            resposta.append({disk.mountpoint: {"perc": psutil.disk_usage(disk.mountpoint).percent, "gb": round(psutil.disk_usage(disk.mountpoint).used / (10**9), 2)}})

        return resposta



# -*- coding: utf-8 -*-

"""Top-level package for Konker Client."""

__author__ = """Gustavo Esteves de Andrade"""
__email__ = 'gandrade@inmetrics.com.br'
__version__ = '0.4.0'

from konker_client.system import System
from konker_client.mqtt import Mqtt
from konker_client.request import Request
import time
import os
import logging
import logging.config
import json
import datetime


FOLDER_LOCATION = os.path.dirname(os.path.abspath(__file__))
FOLDER_LOCATION_LOGS = FOLDER_LOCATION


class KonkerClient():

    __client = None
    device = None
    client_name = None
    service_type = None

    def __init__(self, mode: str, device: str, user: str, password: str, client_name: str, service_type: str):
        self.setup_logging()
        self._logger = logging.getLogger(__name__)

        if mode == 'MQTT':
            self.__client = Mqtt(user, password)
        elif mode == 'REQUEST':
            self.__client = Request(user, password)
        else:
            raise Exception(f"{mode} NOT IMPLEMENTED")

        self.device = device
        self.client_name = client_name
        self.service_type = service_type

    def setup_logging(self, default_path=os.path.join(FOLDER_LOCATION_LOGS, 'logging.json'), default_level=logging.INFO, env_key='LOG_CFG'):
        """
        Função que executa a configuração de logs do Spiderling
        """
        path = default_path
        value = os.getenv(env_key, None)
        if value:
            path = value
        if os.path.exists(path):
            with open(path, 'rt') as f:
                config = json.load(f)
                for key, handler in config['handlers'].items():
                    if 'filename' in handler:
                        try:
                            os.mkdir('logs')
                        except OSError:
                            pass
                        handler['filename'] = os.path.join('logs', handler['filename'])

            logging.config.dictConfig(config)
        else:
            logging.basicConfig(level=default_level)

    def __send_message(self, channel: str, message: dict):
        return self.__client.send_message(channel, message)

    def __send_messages_assync(self, channel: str, list_messages: list):
        return self.__client.send_messages_assync(channel, list_messages)

    def send_business_log_rpa(self, robot_name: str, business_description: str, value: int, business_datetime: str) -> bool:
        message = {
            "LOG_TYPE": "BUSINESS",
            "CLIENT_NAME": self.client_name,
            "SERVICE_TYPE": self.service_type,
            "DATA": {
                "ROBOT_NAME": robot_name,
                "BUSINESS_DESCRIPTION": business_description,
                "VALUE": value,
                "BUSINESS_DATETIME": business_datetime
            }
        }

        return self.__send_message('log', message)

    def send_business_log_qts_caso_teste(self, ct_data: dict) -> bool:
        message = {
            "LOG_TYPE": "BUSINESS",
            "CLIENT_NAME": self.client_name,
            "SERVICE_TYPE": self.service_type,
            "ENTITY_NAME": "CASO_TESTE",
            "DATA": {
                "ID": ct_data.get('id'),
                "NOME": ct_data.get('nome'),
                "FUNCIONALIDADE": ct_data.get('funcionalidade'),
                "DESCRICAO": ct_data.get('descricao'),
                "PRIORIDADE": ct_data.get('prioridade'),
                "CRITICIDADE": ct_data.get('criticidade'),
                "STATUS": ct_data.get('status'),
                "RESPONSAVEL": ct_data.get('responsavel'),
                "ID_SPRINT": ct_data.get('id_sprint'),
                "ID_PROJETO": ct_data.get('id_projeto'),
                "DATA_CRIACAO": ct_data.get('data_criacao'),
            }
        }

        return self.__send_message('log', message)

    def send_business_log_qts_projeto(self, data: dict) -> bool:
        message = {
            "LOG_TYPE": "BUSINESS",
            "CLIENT_NAME": self.client_name,
            "SERVICE_TYPE": self.service_type,
            "ENTITY_NAME": "PROJETO",
            "DATA": {
                "ID": data.get('id'),
                "NOME": data.get('nome'),
            }
        }

        return self.__send_message('log', message)

    def send_business_log_qts_sprint(self, data: dict) -> bool:
        message = {
            "LOG_TYPE": "BUSINESS",
            "CLIENT_NAME": self.client_name,
            "SERVICE_TYPE": self.service_type,
            "ENTITY_NAME": "SPRINT",
            "DATA": {
                "ID": data.get('id'),
                "NOME": data.get('nome'),
                "DATA_INICIAL": data.get('data_inicial'),
                "DATA_FIM": data.get('data_fim'),
                "ID_PROJETO": data.get('id_projeto'),
            }
        }

        return self.__send_message('log', message)

    def send_business_log_qts_caso_teste_execucao(self, data: dict) -> bool:
        message = {
            "LOG_TYPE": "BUSINESS",
            "CLIENT_NAME": self.client_name,
            "SERVICE_TYPE": self.service_type,
            "ENTITY_NAME": "CASO_TESTE_EXECUCAO",
            "DATA": {
                "STATUS": data.get('status'),
                "DURACAO": data.get('duracao'),
                "RESPONSAVEL": data.get('responsavel'),
                "ID_CT": data.get('id_ct'),
                "ID_SPRINT": data.get('id_sprint'),
                "ID_PROJETO": data.get('id_projeto'),
                "DATA_CRIACAO": data.get('data_criacao'),
            }
        }

        return self.__send_message('log', message)

    def send_business_log_qts_defeito(self, data: dict) -> bool:
        message = {
            "LOG_TYPE": "BUSINESS",
            "CLIENT_NAME": self.client_name,
            "SERVICE_TYPE": self.service_type,
            "ENTITY_NAME": "DEFEITO",
            "DATA": {
                "ID": data.get('id'),
                "STATUS": data.get('status'),
                "DATA_CRIACAO": data.get('data_criacao'),
                "CRIADO_POR": data.get('criado_por'),
                "FABRICA_DESENVOLVIMENTO_RESPONSAVEL": data.get('fabrica_desenvolvimento_responsavel'),
                "CAUSA_RAIZ": data.get('causa_raiz'),
                "BUGS_POR_FRENTE": data.get('bugs_por_frente'),
                "TYPE": data.get('type'),
                "NOME": data.get('nome'),
                "ID_SPRINT": data.get('id_sprint'),
                "ID_PROJETO": data.get('id_projeto'),
            }
        }

        return self.__send_message('log', message)

    def send_business_log_caso_teste_defeito(self, data: dict) -> bool:
        message = {
            "LOG_TYPE": "BUSINESS",
            "CLIENT_NAME": self.client_name,
            "SERVICE_TYPE": self.service_type,
            "ENTITY_NAME": "CASO_TESTE_DEFEITO",
            "DATA": {
                "ID_CT": data.get('id_ct'),
                "ID_DEF": data.get('id_def'),
            }
        }

        return self.__send_message('log', message)

    def send_execution_log(self, operation: str, execution_id: int, status: bool, execution_message: str, duration: int) -> bool:
        status = "Sucess" if status else "Failed"
        message = {
            "LOG_TYPE": "EXECUTION",
            "CLIENT_NAME": self.client_name,
            "SERVICE_TYPE": self.service_type,
            "OPERATION": operation,
            "EXECUTION_ID": execution_id,
            "STATUS": status,
            "MESSAGE": execution_message,
            "DURATION": duration
        }

        return self.__send_message('log', message)

    def send_health_log(self) -> bool:
        system = System()
        system.get_health()
        message = {
            "LOG_TYPE": "HEALTH",
            "CLIENT_NAME": self.client_name,
            "SERVICE_TYPE": self.service_type,
            "KONKER_ID": self.device,
            "HEALTH": 1,
            "HOST": {
                "NAME": system.hostname,
                "IP": system.ip,
                "MEMORY": {
                    "PERC": system.host_memory_perc,
                    "USED": system.host_memory_used
                },
                "SWAP": {
                    "PERC": system.host_swap_perc,
                    "USED": system.host_swap_used
                },
                "CPU": system.host_cpu,
                "DISKS": system.host_disks
            },
            "PROCESS": {
                "MEMORY": {
                    "PERC": system.process_memory_perc,
                    "USED": system.process_memory_used,
                },
                "SWAP": {
                    "PERC": system.process_swap_perc,
                    "USED": system.process_swap_used,
                },
                "CPU": system.process_cpu,
            }
        }

        return self.__send_message('health', message)

    def send_acknowledge_log(self, operation: str, execution_id: int) -> bool:
        message = {
            "LOG_TYPE": "ACK",
            "CLIENT_NAME": self.client_name,
            "SERVICE_TYPE": self.service_type,
            "OPERATION": operation,
            "EXECUTION_ID": execution_id,
        }

        return self.__send_message('ack', message)

    def send_report_log(self, execution_id: int, report_message: str) -> bool:
        message = {
            "LOG_TYPE": "REPORT",
            "CLIENT_NAME": self.client_name,
            "SERVICE_TYPE": self.service_type,
            "EXECUTION_ID": execution_id,
            "REPORT_MESSAGE": report_message
        }

        return self.__send_message('report', message)
